/* ***********************************************************
 * This file was automatically generated on 2022-07-26.      *
 *                                                           *
 * LabVIEW Bindings Version 2.1.31                           *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generators git repository on tinkerforge.com       *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Creates beep with configurable frequency
	/// </summary>
	public class BrickletPiezoSpeaker : Device
	{
		/// <summary>
		///  Used to identify this device type in
		///  <see cref="Tinkerforge.IPConnection.EnumerateCallback"/>.
		/// </summary>
		public static int DEVICE_IDENTIFIER = 242;

		/// <summary>
		///  The display name of this device.
		/// </summary>
		public static string DEVICE_DISPLAY_NAME = "Piezo Speaker Bricklet";

		/// <summary>
		///  Function ID to be used with
		///  <see cref="Tinkerforge.Device.GetResponseExpected"/> and
		///  <see cref="Tinkerforge.Device.SetResponseExpected"/>.
		/// </summary>
		public const byte FUNCTION_BEEP = 1;

		/// <summary>
		///  Function ID to be used with
		///  <see cref="Tinkerforge.Device.GetResponseExpected"/> and
		///  <see cref="Tinkerforge.Device.SetResponseExpected"/>.
		/// </summary>
		public const byte FUNCTION_MORSE_CODE = 2;

		/// <summary>
		///  Function ID to be used with
		///  <see cref="Tinkerforge.Device.GetResponseExpected"/> and
		///  <see cref="Tinkerforge.Device.SetResponseExpected"/>.
		/// </summary>
		public const byte FUNCTION_CALIBRATE = 3;

		/// <summary>
		///  Function ID to be used with
		///  <see cref="Tinkerforge.Device.GetResponseExpected"/> and
		///  <see cref="Tinkerforge.Device.SetResponseExpected"/>.
		/// </summary>
		public const byte FUNCTION_GET_IDENTITY = 255;

		private const int CALLBACK_BEEP_FINISHED = 4;

		private const int CALLBACK_MORSE_CODE_FINISHED = 5;


		/// <summary>
		/// </summary>
		public const long BEEP_DURATION_OFF = 0;

		/// <summary>
		/// </summary>
		public const long BEEP_DURATION_INFINITE = 4294967295;


		/// <summary>
		///  This callback is triggered if a beep set by <see cref="Tinkerforge.BrickletPiezoSpeaker.Beep"/> is finished
		/// </summary>
		public event BeepFinishedEventHandler BeepFinishedCallback;
		/// <summary>
		/// </summary>
		public delegate void BeepFinishedEventHandler(BrickletPiezoSpeaker sender);

		/// <summary>
		/// </summary>
		public event BeepFinishedEventHandler BeepFinished // for backward compatibility
		{
			add { BeepFinishedCallback += value; }
			remove { BeepFinishedCallback -= value; }
		}

		/// <summary>
		///  This callback is triggered if the playback of the morse code set by
		///  <see cref="Tinkerforge.BrickletPiezoSpeaker.MorseCode"/> is finished.
		/// </summary>
		public event MorseCodeFinishedEventHandler MorseCodeFinishedCallback;
		/// <summary>
		/// </summary>
		public delegate void MorseCodeFinishedEventHandler(BrickletPiezoSpeaker sender);

		/// <summary>
		/// </summary>
		public event MorseCodeFinishedEventHandler MorseCodeFinished // for backward compatibility
		{
			add { MorseCodeFinishedCallback += value; }
			remove { MorseCodeFinishedCallback -= value; }
		}

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c> and adds  it to
		///  the IPConnection <c>ipcon</c>.
		/// </summary>
		public BrickletPiezoSpeaker(string uid, IPConnection ipcon) : base(uid, ipcon, DEVICE_IDENTIFIER, DEVICE_DISPLAY_NAME)
		{
			apiVersion[0] = 2;
			apiVersion[1] = 0;
			apiVersion[2] = 0;

			callbackWrappers[CALLBACK_BEEP_FINISHED] = new CallbackWrapper(OnBeepFinishedCallback);
			callbackWrappers[CALLBACK_MORSE_CODE_FINISHED] = new CallbackWrapper(OnMorseCodeFinishedCallback);


			responseExpected[FUNCTION_BEEP] = ResponseExpectedFlag.FALSE;
			responseExpected[FUNCTION_MORSE_CODE] = ResponseExpectedFlag.FALSE;
			responseExpected[FUNCTION_CALIBRATE] = ResponseExpectedFlag.ALWAYS_TRUE;
			responseExpected[FUNCTION_GET_IDENTITY] = ResponseExpectedFlag.ALWAYS_TRUE;

			ipcon.AddDevice(this);
		}

		/// <summary>
		///  Beeps with the given frequency for the given duration.
		///  
		///  .. versionchanged:: 2.0.2$nbsp;(Plugin)
		///     A duration of 0 stops the current beep if any, the frequency parameter is
		///     ignored. A duration of 4294967295 results in an infinite beep.
		///  
		///  The Piezo Speaker Bricklet can only approximate the frequency, it will play
		///  the best possible match by applying the calibration (see <see cref="Tinkerforge.BrickletPiezoSpeaker.Calibrate"/>).
		/// </summary>
		public void Beep(long duration, int frequency)
		{
			CheckValidity();

			byte[] request = CreateRequestPacket(14, FUNCTION_BEEP);
			LEConverter.To((int)duration, 8, request);
			LEConverter.To((short)frequency, 12, request);

			SendRequest(request, 0);

		}

		/// <summary>
		///  Sets morse code that will be played by the piezo buzzer. The morse code
		///  is given as a string consisting of "." (dot), "-" (minus) and " " (space)
		///  for *dits*, *dahs* and *pauses*. Every other character is ignored.
		///  
		///  For example: If you set the string "...---...", the piezo buzzer will beep
		///  nine times with the durations "short short short long long long short
		///  short short".
		/// </summary>
		public void MorseCode(string morse, int frequency)
		{
			CheckValidity();

			byte[] request = CreateRequestPacket(70, FUNCTION_MORSE_CODE);
			LEConverter.To((string)morse, 8, 60, request);
			LEConverter.To((short)frequency, 68, request);

			SendRequest(request, 0);

		}

		/// <summary>
		///  The Piezo Speaker Bricklet can play 512 different tones. This function
		///  plays each tone and measures the exact frequency back. The result is a
		///  mapping between setting value and frequency. This mapping is stored
		///  in the EEPROM and loaded on startup.
		///  
		///  The Bricklet should come calibrated, you only need to call this
		///  function (once) every time you reflash the Bricklet plugin.
		///  
		///  Returns *true* after the calibration finishes.
		/// </summary>
		public bool Calibrate()
		{
			CheckValidity();

			byte[] request = CreateRequestPacket(8, FUNCTION_CALIBRATE);

			byte[] response = SendRequest(request, 9);

			return LEConverter.BoolFrom(8, response);
		}

		/// <summary>
		///  Returns the UID, the UID where the Bricklet is connected to,
		///  the position, the hardware and firmware version as well as the
		///  device identifier.
		///  
		///  The position can be 'a', 'b', 'c', 'd', 'e', 'f', 'g' or 'h' (Bricklet Port).
		///  A Bricklet connected to an :ref:`Isolator Bricklet &lt;isolator_bricklet&gt;` is always at
		///  position 'z'.
		///  
		///  The device identifier numbers can be found :ref:`here &lt;device_identifier&gt;`.
		///  |device_identifier_constant|
		/// </summary>
		public override void GetIdentity(out string uid, out string connectedUid, out char position, out byte[] hardwareVersion, out byte[] firmwareVersion, out int deviceIdentifier)
		{
			byte[] request = CreateRequestPacket(8, FUNCTION_GET_IDENTITY);

			byte[] response = SendRequest(request, 33);

			uid = LEConverter.StringFrom(8, response, 8);
			connectedUid = LEConverter.StringFrom(16, response, 8);
			position = LEConverter.CharFrom(24, response);
			hardwareVersion = LEConverter.ByteArrayFrom(25, response, 3);
			firmwareVersion = LEConverter.ByteArrayFrom(28, response, 3);
			deviceIdentifier = LEConverter.UShortFrom(31, response);
		}

		/// <summary>
		/// </summary>
		protected void OnBeepFinishedCallback(byte[] response)
		{
			if (response.Length != 8)
			{
				return; // silently ignoring callback with wrong length
			}

			var handler = BeepFinishedCallback;

			if (handler != null)
			{
				handler(this);
			}
		}

		/// <summary>
		/// </summary>
		protected void OnMorseCodeFinishedCallback(byte[] response)
		{
			if (response.Length != 8)
			{
				return; // silently ignoring callback with wrong length
			}

			var handler = MorseCodeFinishedCallback;

			if (handler != null)
			{
				handler(this);
			}
		}
	}
}
